/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.extension.base;

import org.conqat.ide.commons.ui.databinding.validate.NumberPositiveValidator;
import org.conqat.ide.commons.ui.databinding.validate.TextToDoubleValidator;
import org.conqat.ide.commons.ui.databinding.validate.TextToIntegerValidator;
import org.eclipse.core.databinding.conversion.NumberToStringConverter;
import org.eclipse.core.databinding.conversion.StringToNumberConverter;
import org.eclipse.core.databinding.observable.value.IObservableValue;
import org.eclipse.core.databinding.validation.IValidator;
import org.eclipse.emf.databinding.EMFDataBindingContext;
import org.eclipse.gef.EditPart;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.tabbed.AbstractPropertySection;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.tooling.kernel.ui.util.DataBindingUtils;

/**
 * Base class for property sections, dealing with setting the input and
 * providing some utility methods.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 532E5AECE5CF1BAB9CC2A690D10FAC34
 */
public abstract class PropertySectionBase extends AbstractPropertySection {

	/** Label width. */
	public static final int PROPERTIES_LABEL_WIDTH = 150;

	/** The main composite for the controls. */
	protected Composite composite;

	/** Stores the data binding context used. */
	protected EMFDataBindingContext dbc = new EMFDataBindingContext();

	/** {@inheritDoc} */
	@Override
	public void createControls(Composite parent, TabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);

		composite = getWidgetFactory().createFlatFormComposite(parent);
		composite.setLayout(new GridLayout(2, false));
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		dbc.dispose();
		dbc = null;
		super.dispose();
	}

	/** {@inheritDoc} */
	@Override
	public final void setInput(IWorkbenchPart part, ISelection selection) {
		super.setInput(part, selection);

		if(!(selection instanceof IStructuredSelection)) {
			return;
		}
		Object object = ((IStructuredSelection)selection).getFirstElement();
		if(object instanceof EditPart) {
			object = ((EditPart)object).getModel();
		}
		setSectionInput(object);
	}

	/** This is used to set the single section input. The parameter may be null! */
	protected abstract void setSectionInput(Object input);

	/**
	 * Creates a text widget, which has a {@link ControlDecoration} attached.
	 * The decoration is stored as widget data for convenience.
	 */
	protected Text createDecoratedText(String labelText) {
		Text text = createFormText(labelText);
		text.setData(DataBindingUtils.DECORATION_KEY, new ControlDecoration(text, SWT.LEFT |
				SWT.TOP));
		return text;
	}

	/** Creates form text field. */
	protected Text createFormText(String labelText) {
		Text text = getWidgetFactory().createText(composite, "");
		createFormEntry(text, labelText);
		return text;
	}

	/**
	 * Creates a form entry.
	 */
	protected void createFormEntry(Control control, String labelName) {
		labelName = labelName.trim();
		if(labelName.endsWith(":")) {
			labelName = labelName.replaceAll(":$", "");
		}

		Label label = getWidgetFactory().createLabel(composite, labelName);
		label.moveAbove(control);
		GridDataFactory.defaultsFor(label).hint(PROPERTIES_LABEL_WIDTH, SWT.DEFAULT)
				.align(SWT.BEGINNING, SWT.BEGINNING).applyTo(label);
		GridData gd = GridDataFactory.defaultsFor(control).create();
		gd.heightHint = Math.min(50, gd.heightHint);
		control.setLayoutData(gd);
	}

	/**
	 * {@inheritDoc}
	 * 
	 * <p>
	 * Sub-classes may override, but must call super.refresh() first.
	 */
	@Override
	public void refresh() {
		dbc.dispose();
		dbc = new EMFDataBindingContext();
	}

	/** Binds a positive double value. */
	protected void bindPositiveDoubleValue(Control text, IObservableValue observedValue) {
		bindDoubleValue(text, observedValue, NumberPositiveValidator.INSTANCE);
	}

	/** Binds a double value. */
	protected void bindDoubleValue(Control text, IObservableValue observedValue,
			IValidator numberValidator) {
		DataBindingUtils.performComplexTextBinding(dbc, text, observedValue,
				NumberToStringConverter.fromDouble(false), StringToNumberConverter.toDouble(false),
				TextToDoubleValidator.INSTANCE, numberValidator);
	}

	/** Binds a positive integer value. */
	protected void bindPositiveIntegerValue(Control text, IObservableValue observedValue) {
		bindIntegerValue(text, observedValue, NumberPositiveValidator.INSTANCE);
	}

	/** Binds a integer value. */
	protected void bindIntegerValue(Control text, IObservableValue observedValue,
			IValidator numberValidator) {
		DataBindingUtils.performComplexTextBinding(dbc, text, observedValue,
				NumberToStringConverter.fromInteger(false),
				StringToNumberConverter.toInteger(false), TextToIntegerValidator.INSTANCE,
				numberValidator);
	}
}
